package net.lecnam.info.ex3;

import java.io.IOException;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.lecnam.info.channel.LocalObjectReader;
import net.lecnam.info.channel.LocalObjectWriter;

import net.lecnam.info.channel.ObjectReader;
import net.lecnam.info.channel.ObjectWriter;

public class ConcurrentComputeEngine<T> implements Runnable {

	ObjectReader<Task<T>> in;
	ObjectWriter<T> out;

	ObjectReader<T> inc1;
	ObjectWriter<Task<T>> outc1;

	ObjectReader<T> inc2;
	ObjectWriter<Task<T>> outc2;

	ComputeEngine<T> slave1;
	ComputeEngine<T> slave2;

	Logger logger = Logger.getLogger("Server");

	ConcurrentComputeEngine(ObjectReader<Task<T>> in, ObjectWriter<T> out) {
		this.in = in;
		this.out = out;

		BlockingQueue<Task<T>> bqcs1 = new LinkedBlockingQueue<>();
		BlockingQueue<T> bqsc1 = new LinkedBlockingQueue<>();

		ObjectWriter<T> outs1 = new LocalObjectWriter<>(bqsc1);
		ObjectReader<Task<T>> ins1 = new LocalObjectReader<>(bqcs1);

		slave1 = new ComputeEngine<>(ins1, outs1);

		this.outc1 = new LocalObjectWriter<>(bqcs1);
		this.inc1 = new LocalObjectReader<>(bqsc1);

		BlockingQueue<Task<T>> bqcs2 = new LinkedBlockingQueue<>();
		BlockingQueue<T> bqsc2 = new LinkedBlockingQueue<>();

		ObjectWriter<T> outs2 = new LocalObjectWriter<>(bqsc2);
		ObjectReader<Task<T>> ins2 = new LocalObjectReader<>(bqcs2);

		slave2 = new ComputeEngine<>(ins2, outs2);

		this.outc2 = new LocalObjectWriter<>(bqcs2);
		this.inc2 = new LocalObjectReader<>(bqsc2);

	}

	@Override
	public void run() {
		logger.info("Running slave 1");
		new Thread(slave1).start();
		logger.info("Running slave 2");
		new Thread(slave2).start();
		try {
			boolean done = false;
			while (!done) {
				logger.info("Reading task 1");
				Task<T> task1 = in.readObject();
				outc1.writeObject(task1);
				logger.info("Reading task 2");
				Task<T> task2 = in.readObject();
				outc2.writeObject(task2);
				T result1 = inc1.readObject();
				T result2 = inc2.readObject();
				if (result1 == null) {
					done = true;
				} else {
					out.writeObject(result1);
					out.writeObject(result2);
				}
			}
		} catch (IOException ex) {
			throw new Error(ex);
		}
	}

}
