package net.lecnam.info.ex4;

import java.util.concurrent.Callable;
import java.util.concurrent.Future;
import java.util.concurrent.FutureTask;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.lecnam.info.channel.ObjectReader;
import net.lecnam.info.channel.ObjectWriter;

public class ConcurrentComputeEngine<T> implements Runnable {

	ObjectReader<Task<T>> in;
	ObjectWriter<T> out;

	Logger logger = Logger.getLogger("Server");

	ConcurrentComputeEngine(ObjectReader<Task<T>> in, ObjectWriter<T> out) {
		this.in = in;
		this.out = out;
	}

	static <V> Future<V> futureTask(Callable<V> callable) {
		FutureTask<V> f = new FutureTask(callable);
		new Thread(f).start();
		return f;
	}
	
	@Override
	public void run() {
		try {
			boolean done = false;
			while (!done) {
				logger.info("Reading task 1");
				Task<T> task1 = in.readObject();
				logger.info("Reading task 2");
				Task<T> task2 = in.readObject();
				Future<T> f1 = futureTask(() -> task1.execute());
				Future<T> f2 = futureTask(() -> task2.execute());
				T result1 = f1.get();
				T result2 = f2.get();
				if (result1 == null) {
					done = true;
				} else {
					out.writeObject(result1);
					out.writeObject(result2);
				}
			}
		} catch (Exception ex) {
			throw new Error(ex);
		}
	}

}
