package net.lecnam.info.ex5;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.Future;
import java.util.concurrent.FutureTask;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.lecnam.info.channel.ObjectReader;
import net.lecnam.info.channel.ObjectWriter;

public class ComputeEngine<T> implements Runnable {

	ObjectReader<List<Task<T>>> in;
	ObjectWriter<List<T>> out;

	Logger logger = Logger.getLogger("Server");

	ComputeEngine(ObjectReader<List<Task<T>>> in, ObjectWriter<List<T>> out) {
		this.in = in;
		this.out = out;
	}

	/* Sequential execution
	public List<T> executeTasks(List<Task<T>> tasks) throws IOException {
		List<T> results = new ArrayList<>();
		for (Task<T> task: tasks) {
			results.add(task.execute());
		}
		return results;
	}
	*/
	
	static <V> Future<V> futureTask(Callable<V> callable) {
		FutureTask<V> f = new FutureTask(callable);
		new Thread(f).start();
		return f;
	}
	
	public List<T> executeTasks(List<Task<T>> tasks) throws Exception {
		List<Future<T>> futures = new ArrayList<>();
		for (Task<T> task: tasks) {
			Future<T> future = futureTask(() -> task.execute());
			futures.add(future);
		}
		List<T> results = new ArrayList<>();
		for (Future<T> future: futures) {
			results.add(future.get());
		}
		return results;
	}
	
	@Override
	public void run() {
		logger.info("Running slave 1");
		try {
			boolean done = false;
			while (!done) {
				logger.info("Reading the tasks.");
				List<Task<T>> tasks = in.readObject();
				logger.info("Executing the tasks.");
				if (tasks.isEmpty()) {
					done = true;
				} else {
					List<T> results = executeTasks(tasks);
					out.writeObject(results);
				}
			}
		} catch (Exception ex) {
			throw new Error(ex);
		}
	}

}
